package com.decard.dc_logansdk_android;

import android.os.Looper;
import android.util.Log;
import androidx.annotation.Nullable;

/**
 * 控制台日志格式打印
 *
 * @author Lee
 * @create 2020/4/1 11:46
 */
class LogUtils {

    /**
     * @param tag 传入Class类型 可以用getClass()得到
     * @param out 动态参数，这里是要输出的数据
     */
    protected static void logv(String tag, int line, Object... out) {
        Log.v(getTag(tag, line, out), getMsg(tag, line, out));
    }


    protected static void logd(String tag, int line, Object... out) {
        Log.d(getTag(tag, line, out), getMsg(tag, line, out));
    }


    protected static void logi(String tag, int line, Object... out) {
        Log.i(getTag(tag, line, out), getMsg(tag, line, out));
    }


    protected static void logw(String tag, int line, Object... out) {
        Log.w(getTag(tag, line, out), getMsg(tag, line, out));
    }


    protected static void loge(String tag, int line, Object... out) {
        Log.e(getTag(tag, line, out), getMsg(tag, line, out));
    }


    private static String getTag(String tag, int line, Object... out) {
        //判断是否含有 lambda 表达式
        boolean next = false;
        String fileName;

        StackTraceElement[] s = Thread.currentThread().getStackTrace();
        for (StackTraceElement value : s) {
            if (value.getMethodName().startsWith("lambda")) {
                next = true;
            }
        }

        fileName = s[5 + line].getFileName();

        //得到代码所在的行数
        int lines = s[5 + line].getLineNumber();

        //生成指向java的字符串 加入到TAG标签里面
        // String tAG = "" + "(" + className + ".java:" + lines + ")";

        return "" + "(" + fileName + ":" + lines + ")"+ tag;
    }


    private static String getMsg(String tag, int line, Object... out) {
        //判断是否含有 lambda 表达式
        boolean next = false;
        String methodName;
        String fileName;

        StackTraceElement[] s = Thread.currentThread().getStackTrace();
        for (StackTraceElement value : s) {
            if (value.getMethodName().startsWith("lambda")) {
                next = true;
            }
        }
        if (!next) {
            methodName = s[5 + line].getMethodName();
        } else {
            methodName = s[6 + line].getMethodName();
        }

        String threadMsg = "isMainThread：" + (Thread.currentThread() == Looper.getMainLooper().getThread());

        //生成用户想要输出的数据
        StringBuilder temp;
        temp = new StringBuilder();
        for (Object anOut : out) {
            temp.append("").append(anOut).append(",");
        }
        //删除最后的 ,  号
        if (out.length != 0) {
            temp.deleteCharAt(temp.length() - 1);
        }
        return threadMsg + ", Method Called：" + methodName + "- " + temp;
    }
}