package com.decard.dc_logansdk_android;

import android.content.Context;
import android.os.Environment;
import android.os.RemoteException;
import android.text.TextUtils;
import android.util.Log;
import com.decard.dc_logansdk_android.aidlmanager.AIDLManager;
import com.decard.dc_logansdk_android.aidlmanager.MsgCallBack;
import com.tencent.mars.xlog.Xlog;

import java.io.File;
import java.lang.reflect.Field;
import java.util.Map;
import xcrash.XCrash;

import static com.tencent.mars.xlog.Log.LEVEL_DEBUG;

public class LoganUtils {
    private static final String TAG = "LoganUtils";

    private static LoganUtils loganUtils;
    private static boolean debug;

    public final static String SD_CARD_PATH = Environment.getExternalStorageDirectory() + File.separator + "DCard" + File.separator + "log";
    private AIDLManager aidlManager;
    private Xlog xlog;


    private LoganUtils() {

    }


    public static LoganUtils getInstance() {
        if (loganUtils == null) {
            loganUtils = new LoganUtils();
        }
        return loganUtils;
    }


    /**
     * @param loganUtilsConfig
     */
    public void initLogan(final LoganUtilsConfig loganUtilsConfig) {
        // if (config != null) {
        //     return;
        // }

        Log.d(TAG, "initLogan: path:::" + SD_CARD_PATH + File.separator + loganUtilsConfig.projectNum);

        debug = (boolean) getBuildConfigValue(loganUtilsConfig.context, "DEBUG");

        setLogMars(loganUtilsConfig);

        //SD_CARD_PATH + projectNum + File.separator + "crash"

        loganUtilsConfig.xCrashParams.setLogDir(SD_CARD_PATH + File.separator + loganUtilsConfig.projectNum + File.separator + "crash");

        initXCrash(loganUtilsConfig.context, loganUtilsConfig.xCrashParams);

        aidlManager = new AIDLManager();
        aidlManager.startService(loganUtilsConfig.context, new AIDLManager.ServiceConnectCallBack() {
            @Override public void connect() {
                Log.d(TAG, "服务连接成功 " + System.currentTimeMillis());
                aidlManager.initDCLogSDK(loganUtilsConfig.encryptKey16, loganUtilsConfig.projectNum, loganUtilsConfig.url, loganUtilsConfig.port, loganUtilsConfig.mqttUrl, loganUtilsConfig.deviceSn, new MsgCallBack() {
                    @Override
                    public void response(final Map map) {
                        String logLevel = (String) map.get("logLevel");
                        Log.d(TAG, "修改日志级别操作：" + logLevel);
                        SharedPrefUtils.putString(loganUtilsConfig.context, "logLevel", logLevel);
                        setLogMars(loganUtilsConfig);
                    }
                });
                String logLevel = SharedPrefUtils.getString(loganUtilsConfig.context, "logLevel", "");
                if (TextUtils.isEmpty(logLevel)) {
                    logLevel = String.valueOf(loganUtilsConfig.logLevel);
                }
                //初始化时传入日志等级，上送给服务器
                aidlManager.initProjectNum(logLevel, new MsgCallBack() {
                    @Override public void response(Map map) {

                    }
                });
            }


            @Override public void disConnected() {
                Log.d(TAG, "服务断开连接 ");
            }
        });
    }


    private void setLogMars(LoganUtilsConfig loganUtilsConfig) {
        String logLevel = SharedPrefUtils.getString(loganUtilsConfig.context, "logLevel", "");

        Xlog.isUseMars(true);
        Xlog.setMaxFileSize(70 * 1024);
        Xlog.setMaxAliveTime(7 * 24 * 60 * 60);
        Xlog.appenderOpen(TextUtils.isEmpty(logLevel) ? loganUtilsConfig.logLevel : Integer.parseInt(logLevel), Xlog.AppednerModeAsync, loganUtilsConfig.context.getFilesDir() + "/xlog", SD_CARD_PATH + File.separator + loganUtilsConfig.projectNum, "DECARD", 0,
            "572d1e2710ae5fbca54c76a382fdd44050b3a675cb2bf39feebe85ef63d947aff0fa4943f1112e8b6af34bebebbaefa1a0aae055d9259b89a1858f7cc9af9df1");

        if (debug) {
            Xlog.setConsoleLogOpen(true);
        } else {
            Xlog.setConsoleLogOpen(false);
        }
        xlog = new Xlog();
        com.tencent.mars.xlog.Log.setLogImp(xlog);
    }


    public void flushLog(boolean isSync) {
        xlog.appenderFlush(isSync);
    }


    private static String convertMd532ToMd516(String str) {
        if (str == null || str.length() != 32) { return "";}
        return str.substring(8, 24);
    }


    private void initXCrash(Context context, XCrash.InitParameters parameters) {
        XCrash.init(context, parameters);
    }


    public void test() {
        XCrash.testNativeCrash(true);
        XCrash.testJavaCrash(true);

        XCrash.testNativeCrash(false);
        XCrash.testJavaCrash(false);
    }


    /**
     * TODO 在子模块中获取{@link BuildConfig.DEBUG}值
     *
     * @param context
     * @param fieldName
     * @return
     */
    private static Object getBuildConfigValue(Context context, String fieldName) {
        try {
            Class<?> clazz = Class.forName(context.getPackageName() + ".BuildConfig");
            Field field = clazz.getField(fieldName);
            return field.get(null);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * @param tag
     * @param msg
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void v(String tag, String msg, int... line) {
        int index = 0;
        if (line.length == 1) {
            index = line[0];
        }
        v(tag, msg, 2, true, index + 1);
    }


    /**
     * @param tag
     * @param msg
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void d(String tag, String msg, int... line) {
        int index = 0;
        if (line.length == 1) {
            index = line[0];
        }
        d(tag, msg, 2, true, index + 1);
    }


    /**
     * @param tag
     * @param msg
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void i(String tag, String msg, int... line) {
        int index = 0;
        if (line.length == 1) {
            index = line[0];
        }
        i(tag, msg, 2, true, index + 1);
    }


    /**
     * @param tag
     * @param msg
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void w(String tag, String msg, int... line) {
        int index = 0;
        if (line.length == 1) {
            index = line[0];
        }
        w(tag, msg, 2, true, index + 1);
    }


    /**
     * @param tag
     * @param msg
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void e(String tag, String msg, int... line) {
        int index = 0;
        if (line.length == 1) {
            index = line[0];
        }
        e(tag, msg, 2, true, index + 1);
    }


    /**
     * @param object
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     * {@link LoganUtils#d(String, String, int, boolean, int...)}
     */
    @Deprecated
    public static void d(Class<?> object, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.logd(object.getSimpleName(), index, msg);
            return;
        }
        //        writeLogan(msg, type, isWriteNow);

        com.tencent.mars.xlog.Log.i("mars", msg);
    }


    /**
     * @param tag
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void d(String tag, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.logd(tag, index, msg);
            return;
        }
        //        writeLogan(msg, type, isWriteNow);

        com.tencent.mars.xlog.Log.d(tag, msg);
    }


    /**
     * @param tag
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void v(String tag, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.logv(tag, index, msg);
            return;
        }
        com.tencent.mars.xlog.Log.v(tag, msg);
    }


    /**
     * @param tag
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void i(String tag, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.logi(tag, index, msg);
            return;
        }
        com.tencent.mars.xlog.Log.i(tag, msg);
    }


    /**
     * @param tag
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void w(String tag, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.logw(tag, index, msg);
            return;
        }
        com.tencent.mars.xlog.Log.w(tag, msg);
    }


    /**
     * @param tag
     * @param msg
     * @param type
     * @param isWriteNow
     * @param line 如果要在这层方法之上再封装一层，line传入1,否则不传入
     */
    public static void e(String tag, String msg, int type, boolean isWriteNow, int... line) {
        if (debug) {
            int index = 0;
            if (line.length == 1) {
                index = line[0];
            }
            LogUtils.loge(tag, index, msg);
            return;
        }
        com.tencent.mars.xlog.Log.e(tag, msg);
    }
}
