package com.decard.dc_logansdk_android;

import android.content.Context;
import com.tencent.mars.xlog.Xlog;
import xcrash.XCrash;

public class LoganUtilsConfig {

    private static final long DAYS = 24 * 60 * 60 * 1000; //天
    private static final long M = 1024 * 1024; //M
    private static final long DEFAULT_DAY = 7 * DAYS; //默认删除天数
    private static final long DEFAULT_FILE_SIZE = 1;
    private static final long DEFAULT_MIN_SDCARD_SIZE = 50 * M; //最小的SD卡小于这个大小不写入
    private static final int DEFAULT_QUEUE = 500;

    public static final int LEVEL_ALL = 0;
    public static final int LEVEL_VERBOSE = 0;
    public static final int LEVEL_DEBUG = 1;
    public static final int LEVEL_INFO = 2;
    public static final int LEVEL_WARNING = 3;
    public static final int LEVEL_ERROR = 4;
    public static final int LEVEL_FATAL = 5;
    public static final int LEVEL_NONE = 6;

    String mCachePath; //mmap缓存路径
    String mPathPath; //file文件路径

    long mMaxFile = DEFAULT_FILE_SIZE; //删除文件最大值
    long mDay = DEFAULT_DAY; //删除天数
    long mMaxQueue = DEFAULT_QUEUE;
    long mMinSDCard = DEFAULT_MIN_SDCARD_SIZE; //最小sdk卡大小

    String projectNum;
    String encryptKey16;
    String url = "";
    String port = "";
    String mqttUrl = "";
    Context context;
    String deviceSn = "";
    int logLevel = LEVEL_DEBUG;

    XCrash.InitParameters xCrashParams = null;


    public void setUrl(String url) {
        this.url = url;
    }


    public void setPort(String port) {
        this.port = port;
    }


    public void setMqttUrl(String mqttUrl) {
        this.mqttUrl = mqttUrl;
    }


    public void setContext(Context context) {
        this.context = context.getApplicationContext();
    }


    public void setProjectNum(String projectNum) {
        this.projectNum = projectNum;
    }


    public void setXCrashParams(XCrash.InitParameters params) {
        this.xCrashParams = params;
    }


    public void setEncryptKey16(String encryptKey16) {
        this.encryptKey16 = encryptKey16;
    }


    private LoganUtilsConfig() {
    }


    private void setCachePath(String cachePath) {
        mCachePath = cachePath;
    }

    // private void setPathPath(String pathPath) {
    //     mPathPath = pathPath;
    // }


    private void setMaxFile(long maxFile) {
        mMaxFile = maxFile;
    }


    private void setDay(long day) {
        mDay = day;
    }


    private void setMinSDCard(long minSDCard) {
        mMinSDCard = minSDCard;
    }


    public void setDeviceSn(String deviceSn) {
        this.deviceSn = deviceSn;
    }


    public void setLogLevel(int logLevel) {
        this.logLevel = logLevel;
    }


    public static final class Builder {

        String mCachePath; //mmap缓存路径
        String mPath; //file文件路径
        long mMaxFile = DEFAULT_FILE_SIZE; //删除文件最大值
        long mDay = DEFAULT_DAY; //删除天数

        long mMinSDCard = DEFAULT_MIN_SDCARD_SIZE;

        String projectNum;
        String encryptKey16;
        String url = "";
        String port = "";
        String mqttUrl = "";
        Context context;
        String deviceSn = "";
        int logLevel = Xlog.LEVEL_DEBUG;

        XCrash.InitParameters params;


        public Builder setMqttUrl(String mqttUrl) {
            this.mqttUrl = mqttUrl;
            return this;
        }


        public Builder setXCrashParams(XCrash.InitParameters params) {
            this.params = params;
            return this;
        }


        public Builder setEncryptKey16(String encryptKey16) {
            this.encryptKey16 = encryptKey16;
            return this;
        }


        public Builder setProjectNum(String projectNum) {
            this.projectNum = projectNum;
            return this;
        }


        public Builder setUrl(String url) {
            this.url = url;
            return this;

        }


        public Builder setPort(String port) {
            this.port = port;
            return this;

        }


        public Builder setContext(Context context) {
            this.context = context;
            return this;

        }


        public Builder setCachePath(String cachePath) {
            mCachePath = cachePath;
            return this;
        }


        public Builder setPath(String path) {
            mPath = path;
            return this;
        }


        public Builder setMaxFile(long maxFile) {
            mMaxFile = maxFile;
            return this;
        }


        public Builder setDay(long day) {
            mDay = day * DAYS;
            return this;
        }


        public Builder setMinSDCard(long minSDCard) {
            this.mMinSDCard = minSDCard;
            return this;
        }


        public Builder setDeviceSn(String deviceSn) {
            this.deviceSn = deviceSn;
            return this;
        }


        public Builder setLogLevel(int logLevel) {
            this.logLevel = logLevel;
            return this;
        }


        public LoganUtilsConfig build() {
            LoganUtilsConfig config = new LoganUtilsConfig();
            config.setCachePath(mCachePath);
            // config.setPathPath(mPath);
            config.setMaxFile(mMaxFile);
            config.setMinSDCard(mMinSDCard);
            config.setDay(mDay);
            config.setEncryptKey16(encryptKey16);
            config.setXCrashParams(params);
            config.setProjectNum(projectNum);
            config.setContext(context);
            config.setUrl(url);
            config.setPort(port);
            config.setMqttUrl(mqttUrl);
            config.setDeviceSn(deviceSn);
            config.setLogLevel(logLevel);
            return config;
        }
    }
}
