package com.decard.dc_logansdk_android.aidlmanager;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Binder;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import com.decard.loganservice.ILoganConnect;
import com.decard.loganservice.IMsgCallBack;
import java.util.HashMap;
import java.util.Map;

import static android.content.Context.BIND_AUTO_CREATE;

public class AIDLManager {
    private static final String TAG = "AIDLManager";

    private Context mContext;
    private MsgCallBack mMsgCallBack;

    private ServiceConnectCallBack serviceConnectCallBack;

    private ILoganConnect iLoganConnect;
    private boolean isConnectService;


    public void startService(Context context, ServiceConnectCallBack callback) {
        mContext = context.getApplicationContext();
        serviceConnectCallBack = callback;
        Intent intent = new Intent();
        intent.setClassName("com.decard.loganservice", "com.decard.loganservice.LoganService");
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        isConnectService = mContext.bindService(intent, mServiceConnection, BIND_AUTO_CREATE);
        // mContext.startService(intent);
        Map<String, Boolean> resultMap = new HashMap<>();
    }



    private ServiceConnection mServiceConnection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            iLoganConnect = ILoganConnect.Stub.asInterface(service);
            serviceConnectCallBack.connect();
            try {
                //添加DeathRecipient监听，Binder死亡时候触发
                service.linkToDeath(mDeathRecipient, 0);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }


        @Override
        public void onServiceDisconnected(ComponentName name) {
            serviceConnectCallBack.disConnected();
        }
    };

    private IBinder.DeathRecipient mDeathRecipient = new IBinder.DeathRecipient() {
        @Override
        public void binderDied() {
            if (iLoganConnect == null) {
                return;
            }
            iLoganConnect.asBinder().unlinkToDeath(mDeathRecipient, 0);
            iLoganConnect = null;
            Log.e(TAG, "binderDied: binder 死亡代理 ");
            Intent intent = new Intent();
            intent.setClassName("com.decard.loganservice", "com.decard.loganservice.LoganService");
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            isConnectService = mContext.bindService(intent, mServiceConnection, BIND_AUTO_CREATE);
        }
    };


    @Deprecated
    public void initDevice(String key, String url, String port, MsgCallBack msgCallBack) {
        try {
            if (iLoganConnect != null) {
                iLoganConnect.initDevice(key, url, port, msgCallBack);
            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }


    @Deprecated
    public void initProjectNum(String projectNum, MsgCallBack msgCallBack) {
        try {
            if (iLoganConnect != null) {
                iLoganConnect.initProjectNum(projectNum, msgCallBack);
            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }


    public void initDCLogSDK(String key, String projectNum, String url, String port, String mqttUrl, String deviceSn, IMsgCallBack callback) {
        try {
            if (iLoganConnect != null) {
                iLoganConnect.initDeviceSn(deviceSn);
                iLoganConnect.initDCLogSDK(key, projectNum, url, port, mqttUrl, callback);
            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }

    }


    public interface ServiceConnectCallBack {
        void connect();
        void disConnected();
    }
}
